@@ -20,6 +20,7 @@ from .modules.backend.backend_module import Backend
 
 # Agent-related imports
 from .modules.agents.agents_module import Agents, Agent, AgentsListItem
+from .modules.agents.models.agent import AgentDeploymentType
 
 # Task-related imports
 from .modules.tasks.tasks_module import Tasks, Task, TasksListItem, AgentExecutionStatus
@@ -47,7 +48,7 @@ from xpander_sdk.modules.knowledge_bases.knowledge_bases_module import (
 
 # Configuration and shared models
 from .models.configuration import Configuration
-from .models.shared import OutputFormat
+from .models.shared import OutputFormat, Tokens
 
 __all__ = [
     # xpander.ai Backend
@@ -57,6 +58,7 @@ __all__ = [
     "Agent",
     "AgentsListItem",
     "AgentExecutionStatus",
+    "AgentDeploymentType",
     # Task management
     "Tasks",
     "Task",
@@ -79,5 +81,6 @@ __all__ = [
     # Configuration and shared
     "Configuration",
     "OutputFormat",
-    "User"
+    "User",
+    "Tokens"
 ]
