@@ -10,25 +10,21 @@ from enum import Enum
 from typing import Dict, List, Literal, Optional, Type
 from pydantic import BaseModel, computed_field
 
-from xpander_sdk.models.shared import OutputFormat, XPanderSharedModel
+from xpander_sdk.models.shared import XPanderSharedModel
 from xpander_sdk.modules.tools_repository.models.mcp import MCPServerDetails
 
 
-class AgentDeploymentProvider(str, Enum):
+class AgentDeploymentType(str, Enum):
     """
-    Enumeration of supported deployment providers for agents.
+    Enumeration of supported deployment types for agents.
     
     Values:
-        XPander: Deploy on xpander.ai's managed infrastructure.
-        Worker: Deploy on worker nodes.
-        GpuCloud: Deploy on GPU cloud infrastructure.
-        K8s: Deploy on Kubernetes clusters.
+        Serverless: Serverless agent.
+        Container: Containerized agent.
     """
     
-    XPander = "xpander"
-    Worker = "worker"
-    GpuCloud = "gpuCloud"
-    K8s = "k8s"
+    Serverless = "serverless"
+    Container = "container"
 
 
 class AgentStatus(str, Enum):
@@ -83,7 +79,14 @@ class AgentInstructions(BaseModel):
         Returns:
             List[str]: List of role instructions.
         """
-        return self.role
+        return f"""
+        <instructions>
+            {self.role}
+        </instructions>
+        <goals>
+            {self.goal_str}
+        </goals>
+        """
 
     @computed_field
     @property
