@@ -9,6 +9,7 @@ from __future__ import annotations
 
 import asyncio
 import json
+import os
 import signal
 import sys
 from os import getenv
@@ -22,6 +23,7 @@ from loguru import logger
 from xpander_sdk.core.module_base import ModuleBase
 from xpander_sdk.exceptions.module_exception import ModuleException
 from xpander_sdk.models.configuration import Configuration
+from xpander_sdk.modules.agents.models.agent import SourceNodeType
 from xpander_sdk.modules.tasks.tasks_module import Tasks
 
 from .utils.git_init import configure_git_credentials
@@ -61,10 +63,11 @@ class Events(ModuleBase):
 
     This class manages Server Sent Events (SSE) for real-time task execution requests
     and integrates with agents deployed on xpander.ai. It handles event streaming,
-    retry logic, and background task management.
+    retry logic, and background task management. The worker is directly attached
+    to the agent without a parent worker hierarchy.
 
     Attributes:
-        worker (Optional[DeployedAsset]): Represents the deployed asset/agent.
+        worker (Optional[DeployedAsset]): Represents the deployed asset/agent worker.
         test_task (Optional[LocalTaskTest]): Task to be tested within the local environment.
         configuration (Configuration): SDK configuration with credentials and endpoints.
 
@@ -127,7 +130,6 @@ class Events(ModuleBase):
             thread_name_prefix="xpander-handler",
         )
         self._bg: Set[asyncio.Task] = set()
-        self._root_worker: DeployedAsset | None = None
 
         logger.debug(
             f"Events initialised (base_url={self.configuration.base_url}, "
@@ -142,9 +144,9 @@ class Events(ModuleBase):
         """
         Start the event listener and register handlers for task execution events.
 
-        This method sets up signal handling for graceful shutdown, registers required
-        workers, and begins listening to task execution requests over SSE.
-        use on_task decorator and not the Events module directly.
+        This method sets up signal handling for graceful shutdown, registers the
+        agent worker directly, and begins listening to task execution requests over SSE.
+        Use the @on_task decorator instead of calling this method directly.
 
         Args:
             on_execution_request (ExecutionRequestHandler): Callback handler
@@ -159,10 +161,7 @@ class Events(ModuleBase):
                 sig, lambda s=sig: asyncio.create_task(self.stop(s))
             )
 
-        # Register root worker (blocks until first WorkerRegistration)
-        self._root_worker = await self.register_parent_worker()
-
-        # One SSE consumer per agent
+        # Register agent worker directly
         self.track(
             asyncio.create_task(
                 self.register_agent_worker(self.agent_id, on_execution_request)
@@ -264,7 +263,7 @@ class Events(ModuleBase):
         Args:
             worker_id (str): The unique identifier of the worker to release.
         """
-        url = f"{get_events_base(configuration=self.configuration)}/{worker_id}"
+        url = f"{get_events_base(configuration=self.configuration)}/{worker_id}?type=worker&agent_id={self.agent_id}"
         await self._request_with_retries(
             "POST",
             url,
@@ -279,7 +278,7 @@ class Events(ModuleBase):
         Args:
             worker_id (str): The unique identifier of the worker to update.
         """
-        url = f"{get_events_base(configuration=self.configuration)}/{worker_id}"
+        url = f"{get_events_base(configuration=self.configuration)}/{worker_id}?type=worker&agent_id={self.agent_id}"
         await self._request_with_retries(
             "POST",
             url,
@@ -298,11 +297,14 @@ class Events(ModuleBase):
                 async with httpx.AsyncClient(
                     timeout=None, follow_redirects=True
                 ) as client:
+                    if not url.endswith('/'):
+                        url += "/"
                     async with aconnect_sse(
                         client,
                         "GET",
                         url,
                         headers=get_events_headers(configuration=self.configuration),
+                        follow_redirects=True
                     ) as event_source:
                         async for sse in event_source.aiter_sse():
                             yield sse
@@ -350,6 +352,8 @@ class Events(ModuleBase):
             logger.exception(f"Execution handler failed - {str(e)}")
             error = str(e)
         finally:
+            task_used_tokens = task.tokens
+            task_used_tools = task.used_tools
             await self._release_worker(agent_worker.id)
 
             if error:
@@ -361,13 +365,36 @@ class Events(ModuleBase):
                 task.status = AgentExecutionStatus.Completed
 
             await task.asave()
+            task.tokens = task_used_tokens
+            task.used_tools = task_used_tools
+            
+            if task.tokens:
+                await task.areport_metrics()
 
             logger.info(f"Finished handling task {task.id}")
 
             # local test task, finish? kill the worker
             if self.test_task:
                 logger.info("Local task handled, exiting")
-                sys.exit(0)
+                
+                # Print the task result for CLI
+                if task.result:
+                    logger.info("\n" + "="*50)
+                    logger.info("TASK RESULT:")
+                    logger.info("="*50)
+                    if isinstance(task.result, (dict, list)):
+                        import json
+                        logger.info(json.dumps(task.result, indent=2))
+                    else:
+                        logger.info(task.result)
+                    logger.info("="*50 + "\n")
+                else:
+                    logger.info("\n" + "="*50)
+                    logger.info("TASK COMPLETED (No result set)")
+                    logger.info("="*50 + "\n")
+                
+                # Use os._exit to avoid exception traceback from asyncio
+                os._exit(0)
 
     async def register_agent_worker(
         self,
@@ -381,10 +408,9 @@ class Events(ModuleBase):
             agent_id (str): The unique identifier of the agent to register.
             on_execution_request (ExecutionRequestHandler): The callback function to process task execution requests.
         """
-        assert self._root_worker, "Root worker must be registered first"
         environment = "xpander" if self.is_xpander_cloud else "local"
 
-        url = f"{get_events_base(configuration=self.configuration)}/{self._root_worker.id}/{agent_id}?environment={environment}"
+        url = f"{get_events_base(configuration=self.configuration)}/{agent_id}?environment={environment}"
 
         async for event in self._sse_events_with_retries(url):
             if event.event == EventType.EnvironmentConflict:
@@ -419,7 +445,7 @@ class Events(ModuleBase):
                     )
 
                 if self.test_task:
-                    logger.info(f"Sending test task {self.test_task.model_dump_json()}")
+                    logger.info(f"Invoking agent {self.test_task.model_dump_json()}")
                     created_task = await Tasks(configuration=self.configuration).acreate(
                         agent_id=self.agent_id,
                         prompt=self.test_task.input.text,
@@ -429,7 +455,8 @@ class Events(ModuleBase):
                         worker_id=self.worker.id,
                         output_format=self.test_task.output_format,
                         output_schema=self.test_task.output_schema,
-                        run_locally=True
+                        run_locally=True,
+                        source=SourceNodeType.SDK.value
                     )
                     self.track(
                     asyncio.create_task(
@@ -451,25 +478,6 @@ class Events(ModuleBase):
                     )
                 )
 
-    async def register_parent_worker(self) -> DeployedAsset:
-        """
-        Register the parent worker asset and start receiving events.
-
-        Returns:
-            DeployedAsset: The deployed asset for the root worker after successful registration.
-
-        Raises:
-            RuntimeError: If registration fails due to absence of WorkerRegistration events.
-        """
-        url = get_events_base(configuration=self.configuration)
-
-        async for event in self._sse_events_with_retries(url):
-            if event.event == EventType.WorkerRegistration:
-                return DeployedAsset(**json.loads(event.data))
-
-        raise RuntimeError(
-            "Failed to register root worker – no WorkerRegistration received"
-        )
 
     # --------------------------------------------------------------------- #
     # Misc helpers                                                          #
